#import "CPUController.h"
#import "CPULoadView.h"

#import <ApplicationServices/ApplicationServices.h>

#define LOADVIEW_HEIGHT		65
#define LABELVIEW_HEIGHT	32

#define ICON_HEIGHT	128
#define ICON_WIDTH	128

XloadPreferences * gPreferences;

@implementation CPUController

- (id) init
{
    [super init];

    informationCenter_=[[CPUInformationCenter alloc] init];
    
    numberOfCPU_=[informationCenter_ numberOfCPU];
    
    cpuViewArray_=(CPULoadView **) malloc(numberOfCPU_*sizeof(CPULoadView *));
    cpuNameArray_=(NSTextField **) malloc(numberOfCPU_*sizeof(NSTextField *));
    timer_=nil;
    
    preferences_=[XloadPreferences alloc];
    
    [preferences_ load];
    
    gPreferences=preferences_;

    return self;
}

- (void) dealloc
{
    [informationCenter_ release];
    
    if (timer_!=NULL)
    {
        [timer_ invalidate];
        [timer_ release];
        timer_=NULL;
    }
    
    if (preferences_!=NULL)
    {
        [preferences_ release];
    }

    [super dealloc];
}

- (id) window
{
    return window_;
}

- (unsigned long) numberOfCPU
{
    return numberOfCPU_;
}

- (NSImage *) applicationIcon
{
	return applicationIcon;
}


- (void) updateTimerInterval
{
    if (timer_!=nil)
    {
        [timer_ invalidate];
        [timer_ release];
        timer_=nil;
    }
    
    timer_ = [[NSTimer scheduledTimerWithTimeInterval:[preferences_ samplePeriod] target:
                                        self selector:@selector(updateLoad:)
                                                                         userInfo:nil
                                                                          repeats:YES] retain];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    return YES;
}

- (void)applicationWillTerminate:(NSNotification *)notification
{
    unsigned long i;
    
    for(i=0;i<numberOfCPU_;i++)
    {
        [cpuViewArray_[i] saveViewSettings:i];
    }
    
    [window_ saveFrameUsingName:@"Window Position"];

    [preferences_ save];
    
    // Set the Application Icon back
    
    [NSApp setApplicationIconImage:realApplicationIcon];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
    NSView * contentView= [window_ contentView];
    unsigned long i;
    long verticalPos;
    long horizontalPos=0;
    NSRect nsrect;
    NSRect  screenRect;
    float tLabelHeight;
    
    oldDisplayName_=[preferences_ hideCPUName];
    
    if (oldDisplayName_==NSOffState)
    {
        tLabelHeight=LABELVIEW_HEIGHT;
    }
    else
    {
        tLabelHeight=1;
    }

    [self setWindowPositionAndSize];
    
    screenRect=[[NSScreen mainScreen] visibleFrame];
    
    if ([preferences_ alignment]==kVertical)
    {
         verticalPos=numberOfCPU_*(LOADVIEW_HEIGHT+tLabelHeight)-LOADVIEW_HEIGHT;
    }
    else
    {
         verticalPos=tLabelHeight;
    }

    [window_ setLevel:NSDockWindowLevel];
    
    if ([preferences_ dock]!=NSOffState)
    {
        [window_ orderOut:self];
    }
    else
    {
        [window_ makeKeyAndOrderFront:self];
    }
    
    for(i=0;i<numberOfCPU_;i++)
    {
               
        nsrect=NSMakeRect(horizontalPos+1, verticalPos, LOADVIEW_HEIGHT, LOADVIEW_HEIGHT);
        
        cpuViewArray_[i]=[[CPULoadView alloc] initWithFrame:nsrect];
        [cpuViewArray_[i] loadViewSettings:i];
        [contentView addSubview:cpuViewArray_[i]];
        
        nsrect=NSMakeRect(0, 0, LOADVIEW_HEIGHT, LOADVIEW_HEIGHT);
        [cpuViewArray_[i] setTrackingTag:[cpuViewArray_[i] addTrackingRect:nsrect owner:cpuViewArray_[i] userData:NULL assumeInside:NO]];
        nsrect=NSMakeRect(horizontalPos, verticalPos-LABELVIEW_HEIGHT, LOADVIEW_HEIGHT+2, LABELVIEW_HEIGHT);
        
        
        
        cpuNameArray_[i]=[[NSTextField alloc] initWithFrame:nsrect];
        [cpuNameArray_[i] setEditable:NO];
        [cpuNameArray_[i] setDrawsBackground:NO];
        [cpuNameArray_[i] setBezeled:NO];
        [cpuNameArray_[i] setAlignment:NSCenterTextAlignment];
        
        [cpuNameArray_[i] setStringValue:[informationCenter_ nameOfCPUNumber:i]];
        
        if (oldDisplayName_==NSOffState)
        {
            [contentView addSubview:cpuNameArray_[i]];
        }
        
        if ([preferences_ alignment]==kVertical)
        {
            verticalPos-=(LOADVIEW_HEIGHT+tLabelHeight);
        }
        else
        {
            horizontalPos+=LOADVIEW_HEIGHT+1;
        }

        
    }
    
    timer_ = [[NSTimer scheduledTimerWithTimeInterval:[preferences_ samplePeriod] target:
                                        self selector:@selector(updateLoad:)
                                                                         userInfo:nil
                                                                          repeats:YES] retain];

    [window_ setDelegate:self];
    
    realApplicationIcon=[[NSApp applicationIconImage] copy];
    
    applicationIcon = [[NSImage allocWithZone:[self zone]] initWithSize:NSMakeSize(ICON_HEIGHT, ICON_WIDTH)];
    
    if ([preferences_ dock]!=NSOffState)
    {
        [NSApp setApplicationIconImage:applicationIcon];
    }
}

- (void)drawDockIcon
{
    NSRect aRect;
    float tWidth,tHeight;
    CPULoadView 	* tLoadView;
    unsigned long i;
    
    if ([preferences_ alignment]==kVertical)
    {
        float vOffSet;
        
        tWidth=ICON_WIDTH;
        tHeight=ICON_HEIGHT/numberOfCPU_;
        
        vOffSet=ICON_HEIGHT-tHeight;
        
        for(i=0;i<numberOfCPU_;i++)
        {
            tLoadView=cpuViewArray_[i];
            
            aRect=NSMakeRect(0, vOffSet, tWidth, tHeight);
            
            [tLoadView drawVectorInRect:aRect];
            
            vOffSet-=tHeight;
        }
    }
    else
    {
        float hOffSet;
        
        tWidth=ICON_WIDTH/numberOfCPU_;
        tHeight=ICON_HEIGHT;
        
        hOffSet=0;
        
        for(i=0;i<numberOfCPU_;i++)
        {
            tLoadView=cpuViewArray_[i];
            
            aRect=NSMakeRect(hOffSet, 0, tWidth, tHeight);
            
            [tLoadView drawVectorInRect:aRect];
            
            hOffSet+=tWidth;
        }
    }
}

-(void) updateLoad:(id) sender
{
    unsigned long i;
    
    for(i=0;i<numberOfCPU_;i++)
    {
        [cpuViewArray_[i] addSample:[informationCenter_ loadOfCPUNumber:i]];
    }
    
    if ([preferences_ dock]!=NSOffState)
    {
        [applicationIcon lockFocus];
        
        [self drawDockIcon];
        
        [applicationIcon unlockFocus];
        
        [NSApp setApplicationIconImage:applicationIcon];
    }
}

- (void) showSettings: (id) inSender
{
    [preferences_ showSettings:self];
}

- (void) setWindowPositionAndSize
{
    NSSize size;
    NSRect  screenRect;
    NSPoint point;
    float tLabelHeight;
    
    if ([preferences_ hideCPUName]==NSOffState)
    {
        tLabelHeight=LABELVIEW_HEIGHT;
    }
    else
    {
        tLabelHeight=1;
    }
    
    screenRect=[[NSScreen mainScreen] visibleFrame];
    
    if ([preferences_ alignment]==kVertical)
    {
        size.width=LOADVIEW_HEIGHT+2;
        size.height=numberOfCPU_*(LOADVIEW_HEIGHT+tLabelHeight)+1;
    }
    else
    {
        size.width=numberOfCPU_*(LOADVIEW_HEIGHT+1)+1;
        size.height=LOADVIEW_HEIGHT+tLabelHeight+1;
    }

    if ([preferences_ positionKind] != kCustom)
    {
        switch([preferences_ positionKind])
        {
            case kUpperLeftCorner:
                point.x=screenRect.origin.x;
                point.y=screenRect.origin.y+screenRect.size.height;
                break;
            case kUpperRightCorner:
                point.x=screenRect.origin.x+screenRect.size.width-size.width;
                
                point.y=screenRect.origin.y+screenRect.size.height;

                break;
            case kLowerRightCorner:
                point.x=screenRect.origin.x+screenRect.size.width-size.width;
               	point.y=screenRect.origin.y;
                break;
            case kLowerLeftCorner:
                point.x=screenRect.origin.x;
                point.y=screenRect.origin.y;
                break;
        }
        
                
        [window_ setContentSize:size];
        [window_ setFrameOrigin: point];
        
    }
    else
    {
        [window_ setFrameUsingName:@"Window Position"];
        [window_ setContentSize:size];
    }
}

- (void) setViewPosition
{
    unsigned long i;
    long verticalPos;
    long horizontalPos=0;
    NSRect nsrect;
    float tLabelHeight;
    NSView * contentView= [window_ contentView];
    
    if ([preferences_ hideCPUName]==NSOffState)
    {
        tLabelHeight=LABELVIEW_HEIGHT;
    }
    else
    {
        tLabelHeight=1;
    }


    if ([preferences_ alignment]==kVertical)
    {
         verticalPos=numberOfCPU_*(LOADVIEW_HEIGHT+tLabelHeight)-LOADVIEW_HEIGHT;
    }
    else
    {
         verticalPos=tLabelHeight;
    }
    
    for(i=0;i<numberOfCPU_;i++)
    {
               
        nsrect=NSMakeRect(horizontalPos+1, verticalPos, LOADVIEW_HEIGHT, LOADVIEW_HEIGHT);
        
        [cpuViewArray_[i] setFrameOrigin:nsrect.origin];
        nsrect=NSMakeRect(0, 0, LOADVIEW_HEIGHT, LOADVIEW_HEIGHT);
        [cpuViewArray_[i] removeTrackingRect:[cpuViewArray_[i] trackingTag]];
        [cpuViewArray_[i] setTrackingTag:[cpuViewArray_[i] addTrackingRect:nsrect owner:cpuViewArray_[i] userData:NULL assumeInside:NO]];
        //[cpuViewArray_[i] setNeedsDisplay:YES];
        
        nsrect=NSMakeRect(horizontalPos, verticalPos-LABELVIEW_HEIGHT, LOADVIEW_HEIGHT+2, LABELVIEW_HEIGHT);
        
        
        if ([preferences_ hideCPUName]==NSOffState)
        {
            [cpuNameArray_[i] setFrameOrigin:nsrect.origin];
            
            if (oldDisplayName_==NSOnState)
            {
                // We must show it
                
                
                [contentView addSubview:cpuNameArray_[i]];
            }
            //[cpuNameArray_[i] setNeedsDisplay:YES];
        }
        else
        {
            if (oldDisplayName_==NSOffState)
            {
                // We must hide it
                
                [cpuNameArray_[i] removeFromSuperview];
            }
        }
        
        if ([preferences_ alignment]==kVertical)
        {
            verticalPos-=(LOADVIEW_HEIGHT+tLabelHeight);
        }
        else
        {
            horizontalPos+=LOADVIEW_HEIGHT+1;
        }
    }
    
    oldDisplayName_=[preferences_ hideCPUName];
    
    [contentView setNeedsDisplay:YES];

}

- (void) setVisualizationMode:(int) inMode
{
    if (inMode==NSOffState)
    {
        [NSApp setApplicationIconImage:realApplicationIcon];
        
        [window_ orderFront:self];
        // Show Window
    }
    else
    {
        // HideWindow
        
        [window_ orderOut:self];
        
        [NSApp setApplicationIconImage:applicationIcon];
    }
}

- (void) showHideInDock: (id) inSender
{
    [preferences_ setDock:![preferences_ dock]];
    
    [preferences_ updateUI];
    
    [self setVisualizationMode: [preferences_ dock]];
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    if ([anItem action] == @selector(showHideInDock:))
    {
        if ([preferences_ dock]!=NSOffState)
        {
            [anItem setTitle:NSLocalizedString(@"Display in Window",@"No comment")];
        }
        else
        {
            [anItem setTitle:NSLocalizedString(@"Display in Dock",@"No comment")];
        }
    }
    
    return YES;

}

@end
