/* CPUInformation.m created by stephane on Wed 16-Aug-2000 */

#import "CPUInformation.h"
#include <mach/mach.h>
#include <mach/mach_error.h>
#include <mach/mach_host.h>
#include <mach/message.h>
#include <mach/processor.h>
#include <mach/processor_info.h>

@implementation CPUInformation

- (id) initWithCPU:(processor_basic_info_data_t) inInfo CPUNumber:(unsigned int) inCPUNumber
{
    kern_return_t error;
    processor_basic_info_data_t tInfo;
    processor_cpu_load_info_data_t tLoadInfo;
    processor_flavor_t tFlavor;
    mach_msg_type_number_t tInfoNumber;
    processor_info_array_t infoArray;
    natural_t useless;
    
    CPUNumber_=inCPUNumber;
    
    tInfo=inInfo;
    host_=mach_host_self();

    switch(tInfo.cpu_type)
    {
        case CPU_TYPE_POWERPC:
            switch(tInfo.cpu_subtype)
            {
                case CPU_SUBTYPE_POWERPC_604:
                case CPU_SUBTYPE_POWERPC_604e:
                    CPUName_=[[NSString stringWithCString:"PowerPC 604"] retain];
                    break;
                case CPU_SUBTYPE_POWERPC_750:
                    CPUName_=[[NSString stringWithCString:"PowerPC G3"] retain];
                    break;
                case CPU_SUBTYPE_POWERPC_7400:
                case CPU_SUBTYPE_POWERPC_7450:
                    CPUName_=[[NSString stringWithCString:"PowerPC G4"] retain];
                    break;
#ifdef CPU_SUBTYPE_POWERPC_970
                case CPU_SUBTYPE_POWERPC_970:
#else
                case ((cpu_subtype_t) 100):
#endif /* CPU_SUBTYPE_POWERPC_970 */ 
                    CPUName_=[[NSString stringWithCString:"PowerPC G5"] retain];
                    break;
                default:
                    CPUName_=[[NSString stringWithCString:"PowerPC"] retain];
                    break;
            }
            break;
        case CPU_TYPE_I386:
            switch(tInfo.cpu_subtype)
            {
                default:
                    CPUName_=[[NSString stringWithCString:"Intel"] retain];
                    break;
            }
            break;
        default:
            CPUName_=[[NSString stringWithCString:"Unknown"] retain];
    }
    
    tFlavor=PROCESSOR_CPU_LOAD_INFO;
    
    error = host_processor_info( mach_host_self(), tFlavor, &useless, &infoArray, &tInfoNumber );
    
    tLoadInfo=((processor_cpu_load_info_data_t *) infoArray)[CPUNumber_];
    
    oldCPULoad_[CPU_STATE_USER]=tLoadInfo.cpu_ticks[CPU_STATE_USER];
    oldCPULoad_[CPU_STATE_SYSTEM]=tLoadInfo.cpu_ticks[CPU_STATE_SYSTEM];
    oldCPULoad_[CPU_STATE_IDLE]=tLoadInfo.cpu_ticks[CPU_STATE_IDLE];
    oldCPULoad_[CPU_STATE_NICE]=tLoadInfo.cpu_ticks[CPU_STATE_NICE];
    
    vm_deallocate( mach_task_self(), (vm_address_t)infoArray, tInfoNumber );

    return self;
}

- (NSString *) CPUName
{
    return CPUName_;
}

- (long) CPUFrequency
{
    return CPUFrequency_;
}

- (unsigned long *) CPULoad
{
    processor_cpu_load_info_data_t tLoadInfo;
    mach_msg_type_number_t tInfoNumber;
    processor_flavor_t tFlavor=PROCESSOR_CPU_LOAD_INFO;
    kern_return_t error;
    processor_info_array_t infoArray;
    natural_t useless;

    error = host_processor_info( mach_host_self(), tFlavor, &useless, &infoArray, &tInfoNumber );
    
    tLoadInfo=((processor_cpu_load_info_data_t *) infoArray)[CPUNumber_];


    CPULoad_[CPU_STATE_USER]=tLoadInfo.cpu_ticks[CPU_STATE_USER]-oldCPULoad_[CPU_STATE_USER];
    CPULoad_[CPU_STATE_SYSTEM]=tLoadInfo.cpu_ticks[CPU_STATE_SYSTEM]-oldCPULoad_[CPU_STATE_SYSTEM];
    CPULoad_[CPU_STATE_IDLE]=tLoadInfo.cpu_ticks[CPU_STATE_IDLE]-oldCPULoad_[CPU_STATE_IDLE];
    CPULoad_[CPU_STATE_NICE]=tLoadInfo.cpu_ticks[CPU_STATE_NICE]-oldCPULoad_[CPU_STATE_NICE];
    
    oldCPULoad_[CPU_STATE_USER]=tLoadInfo.cpu_ticks[CPU_STATE_USER];
    oldCPULoad_[CPU_STATE_SYSTEM]=tLoadInfo.cpu_ticks[CPU_STATE_SYSTEM];
    oldCPULoad_[CPU_STATE_IDLE]=tLoadInfo.cpu_ticks[CPU_STATE_IDLE];
    oldCPULoad_[CPU_STATE_NICE]=tLoadInfo.cpu_ticks[CPU_STATE_NICE];

    vm_deallocate( mach_task_self(), (vm_address_t)infoArray, tInfoNumber );
    
    return CPULoad_;
}

@end
