//
//  CPULoadView.m
//  Xload
//
//  Created by stephane on Thu Aug 24 2000.
//  Copyright (c) 2000 Stphane Sudre. All rights reserved.
//

#import "CPULoadView.h"
#import "XloadPreferences.h"

#include <mach/machine.h>

extern XloadPreferences * gPreferences;

static NSMenu * CPULoadViewMenu=nil;
static NSDictionary * CPUFontAttributes=nil;

@implementation CPULoadView

- (id)initWithFrame:(NSRect)frameRect
{
    [super initWithFrame:frameRect];
    
    samplesNumber_=frameRect.size.width-2;
    samplesPresentNumber_=0;
    sampleIndex_=(samplesNumber_-1);
    samples_= (float *) malloc(samplesNumber_*sizeof(float));
    
    totalUser_=0;
    totalKernel_=0;
    totalIdle_=0;
    totalNice_=0;
    totalFull_=0;
    
    for(historyIndex_=0;historyIndex_<HISTORY_SIZE;historyIndex_++)
    {
        history_[historyIndex_].user=0;
        history_[historyIndex_].kernel=0;
        history_[historyIndex_].idle=0;
        history_[historyIndex_].nice=0;
    }
    
    
    showPercentage_=NO;
    historyIndex_=0;

    fill_=YES;
    grid_=NO;
    backColor_=[[[NSColor whiteColor] colorUsingColorSpaceName:NSDeviceRGBColorSpace] retain];
    foreColor_=[[[NSColor redColor] colorUsingColorSpaceName:NSDeviceRGBColorSpace] retain];

    if (CPUFontAttributes==nil)
    {
        NSFont * tFont;
        
        tFont=[NSFont fontWithName:@"Didot" size:18];

        if (tFont!=nil)
        {
            CPUFontAttributes = [[NSDictionary dictionaryWithObject:tFont forKey:NSFontAttributeName] retain];
        }
    }
    
    if (CPULoadViewMenu==nil)
    {
        NSMenuItem * tMenuItem;
        
        CPULoadViewMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
        
        tMenuItem=[[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:NSLocalizedString(@"Filled",@"No comment") action:NULL keyEquivalent:@""];
        [tMenuItem setAction:@selector(invertFill:)];
        [CPULoadViewMenu addItem:tMenuItem];
        [tMenuItem release];
        
        tMenuItem=[[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:NSLocalizedString(@"Show Grid",@"No comment") action:NULL keyEquivalent:@""];
        [tMenuItem setAction:@selector(invertGrid:)];
        [CPULoadViewMenu addItem:tMenuItem];
        [tMenuItem release];
        
         tMenuItem=[[NSMenuItem separatorItem] retain];
        [CPULoadViewMenu addItem:tMenuItem];
        [tMenuItem release];
        
        tMenuItem=[[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:NSLocalizedString(@"BackColor...",@"No comment") action:NULL keyEquivalent:@""];
        [tMenuItem setAction:@selector(backgroundColorSetUp:)];
        [CPULoadViewMenu addItem:tMenuItem];
        [tMenuItem release];
        
        tMenuItem=[[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:NSLocalizedString(@"FillColor...",@"No comment") action:NULL keyEquivalent:@""];
        [tMenuItem setAction:@selector(foregroundColorSetUp:)];
        [CPULoadViewMenu addItem:tMenuItem];
        [tMenuItem release];
    }
    else
    {
        // May be I must retain it
    }
    
    return self;
}

- (void) dealloc
{
    [self removeTrackingRect:trackingTag_];
    
    if (samples_!=NULL)
    {
        free(samples_);
    }
    
    [backColor_ release];
    [foreColor_ release];
    
    [super dealloc];
}

- (NSTrackingRectTag) trackingTag
{
    return trackingTag_;
}

- (void) setTrackingTag:(NSTrackingRectTag) inTrackingTag
{
    trackingTag_=inTrackingTag;
}


- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
    return YES;
}

- (void)mouseEntered:(NSEvent *)theEvent
{
    showPercentage_=YES;
    [self setNeedsDisplay:YES];
}

- (void)mouseExited:(NSEvent *)theEvent
{
    showPercentage_=NO;
    [self setNeedsDisplay:YES];
}


- (BOOL) fill
{
    return fill_;
}

- (void) setFill:(BOOL) inFill
{
    fill_=inFill;
}

- (NSColor *) backColor
{
    return backColor_;
}

- (NSColor *) foreColor
{
    return foreColor_;
}


- (void) addSample:(unsigned long *) inLoad
{
    //
    
    historyIndex_=historyIndex_%HISTORY_SIZE;
    
    totalUser_-=history_[historyIndex_].user;
    totalKernel_-=history_[historyIndex_].kernel;
    totalIdle_-=history_[historyIndex_].idle;
    totalNice_-=history_[historyIndex_].nice;
    
    history_[historyIndex_].user   = inLoad[CPU_STATE_USER];
    history_[historyIndex_].kernel = inLoad[CPU_STATE_SYSTEM];
    history_[historyIndex_].idle   = inLoad[CPU_STATE_IDLE];
    history_[historyIndex_].nice   = inLoad[CPU_STATE_NICE];
    
    historyIndex_++;
    
    totalUser_+=inLoad[CPU_STATE_USER];
    totalKernel_+=inLoad[CPU_STATE_SYSTEM];
    totalIdle_+=inLoad[CPU_STATE_IDLE];
    totalNice_+=inLoad[CPU_STATE_NICE];
    totalFull_=totalUser_+totalKernel_+totalIdle_+totalNice_;

    // Add the value to the Charts Array
    
    sampleIndex_=(sampleIndex_+1)%samplesNumber_;
    samples_[sampleIndex_]=((double)(totalUser_+totalKernel_+totalNice_))/((double)totalFull_);
    samplesPresentNumber_++;
    if (samplesPresentNumber_>samplesNumber_) samplesPresentNumber_=samplesNumber_;
    
    [self setNeedsDisplay: YES];
}

- (void) drawVectorInRect:(NSRect) inRect
{
    float height=inRect.size.height-2;
    float tBottom=inRect.origin.y+1;
    float tWidthRatio=(inRect.size.width-2)/samplesNumber_;
    float tLeft=inRect.origin.x+1;
    long i;
    float tDash[]={2.0,5.0};
    NSBezierPath * tPath;
    
    [[NSColor colorWithCalibratedRed:[backColor_ redComponent] green:[backColor_ greenComponent] blue:[backColor_ blueComponent] alpha:1] set];
    
    NSRectFill(inRect);
    
    [[NSColor blackColor] set];
    tPath=[NSBezierPath bezierPathWithRect:NSMakeRect(inRect.origin.x,inRect.origin.y,inRect.size.width,inRect.size.height)];
    [tPath stroke];
    
    // Draw the load curve
    
    if (grid_==YES)
    {
        NSBezierPath * tDashPath;
        
        [[NSColor colorWithCalibratedRed:0.75 green:0.75 blue:0.75 alpha:1] set];

        // 25 %
        
        tDashPath=[NSBezierPath bezierPath];

        [tDashPath setLineDash:tDash count:4 phase:2];
        
        [tDashPath moveToPoint:NSMakePoint(tLeft,inRect.size.height/4+tBottom)];
        [tDashPath lineToPoint:NSMakePoint(tLeft+inRect.size.width-1,inRect.size.height/4+tBottom)];
        [tDashPath stroke];
        
        // 50 %
        
        [tDashPath removeAllPoints];
        
        [tDashPath moveToPoint:NSMakePoint(tLeft,inRect.size.height/2+tBottom)];
        [tDashPath lineToPoint:NSMakePoint(tLeft+inRect.size.width-1,inRect.size.height/2+tBottom)];
        [tDashPath stroke];
        
        // 75 %
        
        [tDashPath removeAllPoints];
        
        [tDashPath moveToPoint:NSMakePoint(tLeft,(inRect.size.height*3)/4+tBottom)];
        [tDashPath lineToPoint:NSMakePoint(tLeft+inRect.size.width-1,(inRect.size.height*3)/4+tBottom)];
        [tDashPath stroke];
    }
    
    [[NSColor colorWithCalibratedRed:[foreColor_ redComponent] green:[foreColor_ greenComponent] blue:[foreColor_ blueComponent] alpha:1] set];

    
    tPath=[NSBezierPath bezierPath];
    
    if (samplesPresentNumber_>0)
    {
        float tVPos;
    
        if (samplesPresentNumber_<samplesNumber_)
        {
            tVPos=samples_[0]*height+tBottom;
        
            if (fill_==NO)
            {
                [tPath moveToPoint:NSMakePoint(tLeft,tVPos)];
            }
            else
            {
                [tPath moveToPoint:NSMakePoint(tLeft,tBottom)];
                [tPath lineToPoint:NSMakePoint(tLeft,tVPos)];
            }
                        
            for(i=0;i<samplesPresentNumber_;i++)
            {
                tVPos=samples_[i]*height+tBottom;
                
                [tPath lineToPoint:NSMakePoint(i*tWidthRatio+tLeft,tVPos)];
            }
            
            if (fill_==YES)
            {
                i--;
                
                [tPath lineToPoint:NSMakePoint(i*tWidthRatio+tLeft,tBottom)];
                [tPath lineToPoint:NSMakePoint(tLeft,tBottom)];
            }
        }
        else
        {
            long tHPos=tLeft;
            
            if ((sampleIndex_+1)<samplesNumber_)
            {
                tVPos=samples_[sampleIndex_+1]*height+tBottom;
            }
            else
            {
                tVPos=samples_[0]*height+tBottom;
            }
            
            if (fill_==NO)
            {
                [tPath moveToPoint:NSMakePoint(tLeft,tVPos)];
            }
            else
            {
                [tPath moveToPoint:NSMakePoint(tLeft,tBottom)];
                [tPath lineToPoint:NSMakePoint(tLeft,tVPos)];
            }
        
            for(i=sampleIndex_+1;i<samplesNumber_;i++)
            {
                tVPos=samples_[i]*height+tBottom;
                [tPath lineToPoint:NSMakePoint(tHPos*tWidthRatio,tVPos)];
                tHPos++;
            }
            
            for(i=0;i<=sampleIndex_;i++)
            {
               tVPos=samples_[i]*height+tBottom;
               [tPath lineToPoint:NSMakePoint(tHPos*tWidthRatio,tVPos)];
               tHPos++;
            }
            
            if (fill_==YES)
            {
                tHPos--;
                
                [tPath lineToPoint:NSMakePoint(tHPos*tWidthRatio,tBottom)];
                [tPath closePath];
            }
            
        }
        
        if (fill_==YES)
        {
            [tPath fill];
        }
        else
        {
            [tPath stroke];
        }
    }
    
    if ([gPreferences textDock]==NSOnState)
    {
        NSString * tString;
        static char tCString[10];
        short tValue=samples_[sampleIndex_]*100;
        NSSize tSize;
        NSRect aRect;
        
        [[NSColor colorWithCalibratedRed:0.75 green:0.75 blue:0.75 alpha:0.5] set];


        sprintf(tCString,"%hd %%",tValue);
        tString=[NSString stringWithCString:tCString];
        tSize=[tString sizeWithAttributes:CPUFontAttributes];
        
        aRect.origin.x=inRect.origin.x+(inRect.size.width-tSize.width)/2;
        aRect.origin.y=inRect.origin.y+(inRect.size.height-tSize.height)/2;
        aRect.size=tSize;
        
        [tString drawInRect:aRect withAttributes:CPUFontAttributes];
    }


}

- (void) drawRect:(NSRect) aRect
{
    NSGraphicsContext *graphicsContext;
    CGContextRef windowContext;
    unsigned long i;
    float height=aRect.size.height-2;
    CGRect cgRect;float tDash[]={2.0,5.0};
    
    graphicsContext = [NSGraphicsContext currentContext];
    windowContext = [graphicsContext graphicsPort];

    CGContextSetRGBFillColor(windowContext, [backColor_ redComponent],
                                              [backColor_ greenComponent], 
                                              [backColor_ blueComponent], 1);
    //CGContextSetRGBFillColor(windowContext, 0,0,0, 1);
    
    NSRectFill(aRect);
    
    CGContextSetRGBStrokeColor(windowContext, 0,0,0, 1);
    
    //CGContextSetRGBStrokeColor(windowContext, 0,0.5,0, 1);
    
    CGContextSetLineWidth(windowContext,1.0);
    
    
    CGContextBeginPath(windowContext);
    
    cgRect=CGRectMake(aRect.origin.x,aRect.origin.y,aRect.size.width,aRect.size.height);
    CGContextAddRect(windowContext,cgRect);
    CGContextClosePath(windowContext);
    CGContextStrokePath(windowContext);
    
    
    // Draws the main % lines
    
    if (grid_==YES)
    {
        CGContextSetRGBStrokeColor(windowContext, 0.75,0.75,0.75, 1);
        CGContextSetLineDash(windowContext, 4, tDash, 2);
        
        // 25 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,aRect.size.height/4);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,aRect.size.height/4);
        CGContextClosePath(windowContext);
        CGContextStrokePath(windowContext);
        
        
        // 50 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,aRect.size.height/2);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,aRect.size.height/2);
        CGContextClosePath(windowContext);
        CGContextStrokePath(windowContext);
        
        
        // 75 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,(aRect.size.height*3)/4);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,(aRect.size.height*3)/4);
        CGContextClosePath(windowContext);
        CGContextStrokePath(windowContext);
        
        CGContextSetLineDash(windowContext, 0, NULL, 0);
        
        /*CGContextSetRGBStrokeColor(windowContext, 0,0.50,0, 1);
        
        // 25 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,aRect.size.height/4);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,aRect.size.height/4);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);
        
        // 50 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,aRect.size.height/2);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,aRect.size.height/2);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);
        
        // 75 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,1,(aRect.size.height*3)/4);
        CGContextAddLineToPoint(windowContext,aRect.size.width-1,(aRect.size.height*3)/4);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);
        
        // 25 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,(aRect.size.width)/4,aRect.size.height);
        CGContextAddLineToPoint(windowContext,(aRect.size.width)/4,1);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);
        
        // 50 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,(aRect.size.width)/2,aRect.size.height);
        CGContextAddLineToPoint(windowContext,(aRect.size.width)/2,1);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);
        
        // 75 %
        
        CGContextBeginPath(windowContext);
        CGContextMoveToPoint(windowContext,(aRect.size.width*3)/4,aRect.size.height);
        CGContextAddLineToPoint(windowContext,(aRect.size.width*3)/4,1);
        CGContextStrokePath(windowContext);
        CGContextClosePath(windowContext);*/

    }
    
    // Draw the load curve
    
    CGContextSetRGBFillColor(windowContext, [foreColor_ redComponent],
                                            [foreColor_ greenComponent], 
                                            [foreColor_ blueComponent], 1);
    CGContextSetRGBStrokeColor(windowContext, [foreColor_ redComponent],
                                              [foreColor_ greenComponent], 
                                              [foreColor_ blueComponent], 1);

    CGContextBeginPath(windowContext);
    
    if (samplesPresentNumber_>0)
    {
        float tVPos;
    
        if (samplesPresentNumber_<samplesNumber_)
        {
            tVPos=samples_[0]*height+1;
        
            if (fill_==NO)
            {
                CGContextMoveToPoint(windowContext,1,tVPos);
            }
            else
            {
                CGContextMoveToPoint(windowContext,1,1);
                CGContextAddLineToPoint(windowContext,1,tVPos);
            }
                        
            for(i=0;i<samplesPresentNumber_;i++)
            {
                tVPos=samples_[i]*height+1;
                CGContextAddLineToPoint(windowContext,i+1,tVPos);
            }
            
            if (fill_==YES)
            {
                CGContextAddLineToPoint(windowContext,i,1);
                CGContextAddLineToPoint(windowContext,1,1);
            }
        }
        else
        {
            long tHPos=1;
            
            if ((sampleIndex_+1)<samplesNumber_)
            {
                tVPos=samples_[sampleIndex_+1]*height+1;
            }
            else
            {
                tVPos=samples_[0]*height+1;
            }
            
            if (fill_==NO)
            {
                CGContextMoveToPoint(windowContext,1,tVPos);
            }
            else
            {
                CGContextMoveToPoint(windowContext,1,1);
                CGContextAddLineToPoint(windowContext,1,tVPos);
            }
        
            for(i=sampleIndex_+1;i<samplesNumber_;i++)
            {
                tVPos=samples_[i]*height+1;
                CGContextAddLineToPoint(windowContext,tHPos,tVPos);
                tHPos++;
            }
            
            for(i=0;i<=sampleIndex_;i++)
            {
               tVPos=samples_[i]*height+1;
               CGContextAddLineToPoint(windowContext,tHPos,tVPos);
               tHPos++;
            }
            
            if (fill_==YES)
            {
                CGContextAddLineToPoint(windowContext,tHPos-1,1);
                CGContextAddLineToPoint(windowContext,1,1);
            }
        }
        
        
        
        if (fill_==YES)
        {
            CGContextClosePath(windowContext);
            CGContextDrawPath(windowContext,kCGPathFill);
        }
        else
        {
            CGContextStrokePath(windowContext);
        }
    }
    

    
    // Drawing of the Percentage
    
    if (showPercentage_==YES)
    {
        NSString * tString;
        static char tCString[10];
        short tValue=samples_[sampleIndex_]*100;
        NSSize tSize;
        
        
        sprintf(tCString,"%hd %%",tValue);
        tString=[NSString stringWithCString:tCString];
        tSize=[tString sizeWithAttributes:CPUFontAttributes];
        
        aRect.origin.x=aRect.origin.x+(aRect.size.width-tSize.width)/2;
        aRect.origin.y=aRect.origin.y+(aRect.size.height-tSize.height)/2;
        aRect.size=tSize;
        
        [tString drawInRect:aRect withAttributes:CPUFontAttributes];
    }
}

- (void) mouseDown:(NSEvent *)theEvent
{
    if ([theEvent clickCount]==2)
    {
        [[self window] miniaturize:self];
    }

}

- (NSMenu *) menu
{
   id tMenuItem;
    
    tMenuItem=[CPULoadViewMenu itemAtIndex:0];
	
    if (fill_==YES)
    {
        [tMenuItem setState:NSOnState];
    }
    else
    {
        [tMenuItem setState:NSOffState];
    }
    
     tMenuItem=[CPULoadViewMenu itemAtIndex:1];
	 
    if (grid_==YES)
    {
        [tMenuItem setTitle:NSLocalizedString(@"Hide Grid",@"No comment")];
    }
    else
    {
        [tMenuItem setTitle:NSLocalizedString(@"Show Grid",@"No comment")];
    }

    
    return CPULoadViewMenu;
}

- (void) invertFill: (id) inSender
{
    fill_=!fill_;
    [self setNeedsDisplay:YES];
}

- (void) invertGrid: (id) inSender
{
    grid_=!grid_;
    [self setNeedsDisplay:YES];
}


- (void) backgroundColorSetUp: (id) inSender
{
    NSColorPanel * tColorPanel;
    
    tColorPanel = [NSColorPanel sharedColorPanel];
    [tColorPanel setTarget:self];
    [tColorPanel setAction:@selector(setBackgroundColor:)];
    
    [tColorPanel setColor:backColor_];
    
    [tColorPanel makeKeyAndOrderFront:self];
    
}

- (void) foregroundColorSetUp: (id) inSender
{
    NSColorPanel * tColorPanel;
    
    tColorPanel = [NSColorPanel sharedColorPanel];
    [tColorPanel setTarget:self];
    [tColorPanel setAction:@selector(setForegroundColor:)];
    
    [tColorPanel setColor:foreColor_];
    
    
    [tColorPanel makeKeyAndOrderFront:self];

}

- (void) setBackgroundColor: (id) inSender
{
    [backColor_ autorelease];
    backColor_=[[inSender color] copy];
    [self setNeedsDisplay:YES];
}

- (void) setForegroundColor: (id) inSender
{
    [foreColor_ autorelease];
    foreColor_=[[inSender color] copy];
    [self setNeedsDisplay:YES];
}

- (void) loadViewSettings:(int) inIndex
{
    NSUserDefaults * tDefault;
    NSString * tStringIndex;
    NSNumber * tNumber;
    
    tDefault=[NSUserDefaults standardUserDefaults];

    tStringIndex=[[NSNumber numberWithInt:inIndex] stringValue];
    
    tNumber=[tDefault objectForKey:[@"Fill " stringByAppendingString:tStringIndex]];
    
    if (tNumber!=nil)
    {
        fill_=[tNumber boolValue];
        
        tNumber=[tDefault objectForKey:[@"Grid " stringByAppendingString:tStringIndex]];
        grid_=[tNumber boolValue];
        
        [backColor_ release];
        backColor_=[[CPULoadView stringToColor:[tDefault stringForKey:[@"BackColor " stringByAppendingString:tStringIndex]]] retain];

        [foreColor_ release];
        
        // Fix for the bug
        
        if ([tDefault stringForKey:[@"ForeColor " stringByAppendingString:tStringIndex]]==nil)
        {
            foreColor_=[[CPULoadView stringToColor:[tDefault stringForKey:[@"ForeColor" stringByAppendingString:tStringIndex]]] retain];
        }
        else
        {
            foreColor_=[[CPULoadView stringToColor:[tDefault stringForKey:[@"ForeColor " stringByAppendingString:tStringIndex]]] retain];
        }
    }
}

- (void) saveViewSettings:(int) inIndex
{
    NSUserDefaults * tDefault;
    NSString * tStringIndex;
    NSNumber * tNumber;
    
    tDefault=[NSUserDefaults standardUserDefaults];

        
    tStringIndex=[[NSNumber numberWithInt:inIndex] stringValue];
    
    tNumber=[NSNumber numberWithBool:fill_];
    [tDefault setObject:tNumber forKey:[@"Fill " stringByAppendingString:tStringIndex]];
    
    
    
    tNumber=[NSNumber numberWithBool:grid_];
    [tDefault setObject:tNumber forKey:[@"Grid " stringByAppendingString:tStringIndex]];
    
    [tDefault setObject:[CPULoadView colorToString:backColor_] forKey:[@"BackColor " stringByAppendingString:tStringIndex]];
    
    [tDefault setObject:[CPULoadView colorToString:foreColor_] forKey:[@"ForeColor " stringByAppendingString:tStringIndex]];
    
    [tDefault synchronize];
    
}

// This code was borrowed from CPU.

+ (NSString *)colorToString:(NSColor *)theColor
{
    NSColor *tempColor = [theColor colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];

    return([NSString stringWithFormat:@"[%f] [%f] [%f]"
                                   ,[tempColor redComponent]
                                   ,[tempColor greenComponent]
                                   ,[tempColor blueComponent]]);
}

+ (NSColor *)stringToColor:(NSString *)theString
{
    float r,g,b;

    sscanf([theString cString],"[%f] [%f] [%f]",&r,&g,&b);
    return([NSColor colorWithCalibratedRed:r green:g blue:b alpha:1.0]);
}


@end
