//
//  XloadPreferences.m
//  XLoad
//
//  Created by stephane on Mon Oct 02 2000.
//  Copyright (c) 2000 Stphane Sudre. All rights reserved.
//

#import "XloadPreferences.h"
#import "CPUController.h"

@implementation XloadPreferences

- (void) awakeFromNib
{
	[IBwindow_ setExcludedFromWindowsMenu:YES];
		
	[IBwindow_ setMenu:nil];
   
	[IBwindow_ center];
	
	if ([CPUcontroller_ numberOfCPU]==1)
	{
		[IBorientation_ setEnabled:NO];
	}
}

#pragma mark -

- (int) dock
{
    return dock_;
}

- (void) setDock:(int) inDock;
{
    dock_=inDock;
}

- (int) textDock
{
    return textDock_;
}

- (void) setTextDock:(int) inTextDock
{
    textDock_=inTextDock;
}

- (int) hideCPUName
{
    return hideCPUName_;
}

- (void) setHideCPUName:(int) inHideCPUName;
{
    hideCPUName_=inHideCPUName;
}

- (int) positionKind
{
    return positionKind_;
}

- (void) setPositionKind:(int) inPositionKind
{
    positionKind_=inPositionKind;
}

- (NSRect) position
{
    return position_;
}

- (void) setPosition:(NSRect) inPosition
{
    position_=inPosition;
}

- (int) alignment
{
    return alignment_;
}

- (void) setAlignment:(int) inAlignment
{
    alignment_=inAlignment;
}

- (NSTimeInterval) samplePeriod;
{
    return samplePeriod_;
}

- (void) setSamplePeriod:(NSTimeInterval) inSamplePeriod
{
    samplePeriod_=inSamplePeriod;
}

#pragma mark -

- (void) load
{
    NSUserDefaults * tDefault;
    NSString * tString;
    
    tDefault=[NSUserDefaults standardUserDefaults];
        
    tString=[tDefault objectForKey:@"Position Kind"];
    if (tString!=nil)
    {
        positionKind_=[tString intValue];
        
        alignment_=[tDefault integerForKey:@"Alignment"];
        
        samplePeriod_=[tDefault floatForKey:@"Sample Period"];
        
        // Version 1.2
        
        tString=[tDefault objectForKey:@"Hide CPU Name"];
        if (tString!=nil)
        {
            hideCPUName_=[tString intValue];
        }
        else
        {
            hideCPUName_=NSOffState;
        }
        
        // Version 1.3
        
        tString=[tDefault objectForKey:@"Show in Dock"];
        if (tString!=nil)
        {
            dock_=[tString intValue];
        }
        else
        {
            dock_=NSOffState;
        }
        
        // Version 1.3.1
        
        tString=[tDefault objectForKey:@"Show text in Dock"];
        if (tString!=nil)
        {
            textDock_=[tString intValue];
        }
        else
        {
            textDock_=NSOffState;
        }
    }
    else
    {	
        [self setDefault];
    }
}

- (void) save
{
    NSUserDefaults * tDefault;
    
    tDefault=[NSUserDefaults standardUserDefaults];
    
    [tDefault setInteger:positionKind_ 	forKey:@"Position Kind"];
    [tDefault setInteger:alignment_ 	forKey:@"Alignment"];
    [tDefault setFloat:samplePeriod_ 	forKey:@"Sample Period"];
    [tDefault setInteger:hideCPUName_ 	forKey:@"Hide CPU Name"];
    [tDefault setInteger:dock_ 	forKey:@"Show in Dock"];
    [tDefault setInteger:textDock_ 	forKey:@"Show text in Dock"];
}

- (void) setDefault
{
    positionKind_=kUpperRightCorner;
    
    alignment_=kVertical;
    samplePeriod_=0.5;
    hideCPUName_=NSOffState;
    dock_=NSOffState;
}

#pragma mark -

- (void) showSettings: (id) sender
{
     CPUcontroller_=sender;
     
     if (!IBposition_)
	 {
        if (![NSBundle loadNibNamed:@"Preferences" owner:self]) 
		{
            NSLog(@"Failed to load Preferences.nib");
			
            NSBeep();
            
			return;
        }
    }
	
	[self updateUI];
	
    [IBwindow_ makeKeyAndOrderFront:nil];
}

- (void) updateUI
{
    [IBposition_ selectCellWithTag:positionKind_];
    [IBorientation_ selectCellWithTag:alignment_];
    [IBsamplePeriod_ setFloatValue:samplePeriod_];
    [IBcpuName_ setState:hideCPUName_];
    [IBdock_ setState:dock_];
    
    if (dock_==NSOffState)
    {
        [IBtextDock_ setEnabled:NO];
        [IBtextDock_ setState:textDock_];
    }
}

- (void) setSettings:(id) sender
{
    int oldPositionKind=positionKind_;
    int oldAlignment=alignment_;
    int oldHideCPUName=hideCPUName_;
    int oldDock=dock_;
        
    positionKind_=[[IBposition_ selectedCell] tag];
    alignment_=[[IBorientation_ selectedCell] tag];
    
    samplePeriod_=[IBsamplePeriod_ floatValue];
    hideCPUName_=[IBcpuName_ state];
    
    dock_=[IBdock_ state];
    textDock_=[IBtextDock_ state];
     
    if (sender == IBdock_)
    {
       [IBtextDock_ setEnabled:([IBdock_ state]==NSOffState) ? NO:YES];
    }
    
    if (oldAlignment!=alignment_       ||
        oldPositionKind!=positionKind_ ||
        oldHideCPUName!=hideCPUName_)
    {
        [IBwindow_ saveFrameUsingName:@"Window Position"];
        [CPUcontroller_ setWindowPositionAndSize];
        [CPUcontroller_ setViewPosition];
    }
    
    if (oldDock!=dock_)
    {
        [CPUcontroller_ setVisualizationMode: dock_];
    }
    
    [CPUcontroller_ updateTimerInterval];
    
    [self save];

}

@end
